/* jsync.c: Julians csync stuff */

/*
 * $Header: /a/victor/nexor/user4/jpo/xemp/xemp5.0/lib/csync/RCS/jsync.c,v 1.3 1996/07/19 08:23:49 jpo Exp $
 *
 * $Log: jsync.c,v $
 * Revision 1.3  1996/07/19 08:23:49  jpo
 * More error checking
 *
 * Revision 1.2  1996/07/17 06:51:18  jpo
 * New partial parsing format
 *
 */

#include <stdio.h>
#include <sys/types.h>
#include <string.h>
#include <stddef.h>
#include "toolkit.h"
#include "jsync.h"

Object *Olist;
int csync_debug = 1;
extern double atof _PROTO((char *cp));

static void parse_object _PROTO((int code, char *str));
static int parse_typedef _PROTO((char *str));
static int getnum _PROTO((char **str));


CMD_TABLE type_tbl[] = {
    {"char", EDB_CHAR },
    {"uchar", EDB_UCHAR},
    {"short", EDB_SHORT},
    {"ushort", EDB_USHORT},
    {"int", EDB_INT},
    {"long", EDB_LONG},
    {"float", EDB_FLOAT},
    {"double", EDB_DOUBLE},
    {"time", EDB_TIME},
    { NULL, EDB_BAD },
};

CMD_TABLE edb_tbl[] = {
    {"version", EDB_VERSION},
    {"checksum", EDB_CHECKSUM},
    {"type def", EDB_TYPE_DEF},
    {"update", EDB_UPDATE},
    {"sector class", SCT_CLASS},
    {"sector object", SCT_OBJECT},
    {"ship class", SHP_CLASS},
    {"ship object", SHP_OBJECT},
    {"plane class", PLN_CLASS},
    {"plane object", PLN_OBJECT},
    {"land class", LND_CLASS},
    {"land object", LND_OBJECT},
    {"nuke class", NUK_CLASS},
    {"nuke object", NUK_OBJECT},
    {"news class", NWS_CLASS},
    {"news object", NWS_OBJECT},
    {"treaty class", TRT_CLASS},
    {"treaty object", TRT_OBJECT},
    {"trade class", TRD_CLASS},
    {"trade object", TRD_OBJECT},
    {"power class", PWR_CLASS},
    {"power object", PWR_OBJECT},
    {"nation class", NAT_CLASS},
    {"nation object", NAT_OBJECT},
    {"loan class", LOA_CLASS},
    {"loan object", LOA_OBJECT},
    {"map class", MAP_CLASS},
    {"map object", MAP_OBJECT},
    {"bmap class", BMP_CLASS},
    {"bmap object", BMP_OBJECT},
    {"com class", COM_CLASS},
    {"com object", COM_OBJECT},
    {"item class", ITM_CLASS},
    {"item object", ITM_OBJECT},
    {"product class", PRD_CLASS},
    {"product object", PRD_OBJECT},
    {"version class", VER_CLASS},
    {"version object", VER_OBJECT},
    {"general class", GEN_CLASS},
    {"general object", GEN_OBJECT},
    { NULL, 0},
};

Name2Mapping *namemap[] = {
    &nation_map,
    &prod_class_map,
    &item_class_map,
    &plane_class_map,
    &plane_obj_map,
    &land_class_map,
    &land_obj_map,
    &ship_class_map,
    &ship_obj_map,
    &nuke_class_map,
    &nuke_obj_map,
    &sector_obj_map,
    &sector_class_map,
    &map_obj_map,
    &bmap_obj_map,
    NULL,
};

const Mapping *find_mapname (str, mp)
const char *str;
const Mapping *mp;
{
    while (mp -> pc_name) {
	if (strcasecmp (mp -> pc_name, str) == 0)
	    return mp;
	mp ++;
    }
    return NULL;
}
    

Name2Mapping *find_map (code)
int code;
{
    Name2Mapping **mp;

    for (mp = namemap; *mp; mp ++)
	if ((*mp) -> code == code)
	    return *mp;
    return NULL;
}

const char *rlookup(n, tbl)
int n;
CMD_TABLE tbl[];
{
    CMD_TABLE *tp;

    for (tp = tbl; tp -> tb_name; tp ++)
	if (tp -> tb_value == n)
	    return tp -> tb_name;
    return NULL;
}

static char *getname (str)
char **str;
{
    char *p = *str, *cp;
    static char buf[512];
    
    for (cp = buf; *p && (*p != ':' && *p != ','); p++) {
	if (*p == '^')
	    *cp ++ = ':';
	else *cp ++ = *p;
    }
    *cp = 0;
    *str = p;
    return buf;
}

Object *find_object(type)
int type;
{
    Object *op;

    for (op = Olist; op; op = op->next)
	if (op -> otype == type)
	    return op;
    return NULL;
}

void dump_object (fp)
FILE *fp;
{
    Object *op;
    Field *f;

    for (op = Olist; op; op = op->next) {
	fprintf (fp, "%d %d %d ", EDB_TYPE_DEF, op -> btype, op -> otype);
	for (f = op -> field; f; f = f -> next) {
	    fprintf(fp, "%s,%d", f -> fname, f -> ftype);
	    if (f -> fsize)
		fprintf (fp, ",%d", f -> fsize);
	    putc (':', fp);
	}
	putc ('\n', fp);
    }
}

static Object *new_object (type)
int type;
{
    Object *obj = (Object *)malloc (sizeof *obj);
    obj -> otype = type;
    obj -> next = Olist;
    obj -> field = NULL;
    Olist = obj;
    return obj;
}

static void addfield (obj, name, type, fsize)
Object *obj;
char *name;
int type;
int fsize;
{
    Field **fpp, *fp;

    for (fpp = &obj -> field; *fpp; fpp = &(*fpp)->next)
	continue;
    *fpp = fp = (Field *) malloc (sizeof **fpp);
    fp -> fname = strdup(name);
    fp -> ftype = type;
    fp -> fsize = fsize;
    fp -> next = NULL;
}

void edb_parse_c_sync (str)
char *str;
{
    int code;

    code = getnum(&str);
    if (code == EDB_TYPE_DEF)
	parse_typedef (str);
    else
	parse_object (code, str);
}

const char *edb_c_sync_type (str)
char *str;
{
    int code = getnum(&str);
    const char *cp;

    if (code == EDB_TYPE_DEF) {
	code = getnum (&str);	/* base code */
	code = getnum (&str);	/* real code */
    }
    if (cp = rlookup (code, edb_tbl))
	return cp;
    else return "<UNKNOWN>";
}

static void updatefield (np, fp, mp, ptr, offst, cp)
Name2Mapping *np;
Field *fp;
Mapping *mp;
void *ptr;
int offst;
char *cp;
{
    long emp_l = 0;
    double emp_d = 0;
    char *emp_p;
    time_t emp_t = 0;

    if (csync_debug > 2)
	printf ("Update: %s field %s %s", np -> name,
		fp -> fname, cp);
    if (mp) {
	char *pp;
	if (csync_debug > 2)
	    printf (" number %d, type %d, offset %d",
		    mp -> xe_number, mp -> xe_type, mp -> xe_offset);
	pp = ptr;
	pp += mp -> xe_offset;
	emp_p = cp;
	switch (fp -> ftype) {
	case EDB_CHAR:
	    if (fp -> fsize > 1) {
		break;
	    }
	    /* else fall */
	case EDB_UCHAR:
	case EDB_SHORT:
	case EDB_USHORT:
	case EDB_INT:
	case EDB_LONG:
	    emp_l = atoi(cp);
	    emp_d = emp_t =  emp_l;
	    break;
	case EDB_TIME:
	    emp_t = atoi(cp);
	    emp_d = emp_l = emp_t;
	    break;
	case EDB_FLOAT:
	case EDB_DOUBLE:
	    emp_d = atof(cp);
	    emp_t = emp_l = emp_d;
	    break;
	}
	switch (mp -> xe_type) {
	case EDB_STRARRY:
	{
	    char **cpp = (char **)(pp + sizeof(char *) * offst);
	    strcpy(*cpp, emp_p);
	    break;
	}
	case EDB_STRING:
	{
	    char **cpp = (char **)(pp + sizeof(char *) * offst);
	    if (*cpp) free (*cpp);
	    *cpp = strdup (emp_p);
	    break;
	}
	case EDB_CHAR:
	{
	    char *p = pp + sizeof(char) * offst;
	    *p = emp_l;
	    break;
	}
	case EDB_UCHAR:
	{
	    unsigned char *p = (unsigned char *)(pp + 
					 sizeof(unsigned char) * offst);
	    *p = emp_l;
	    break;
	}
	case EDB_SHORT:
	{
	    short *ip = (short *)(pp + sizeof(short) * offst);
	    *ip = emp_l;
	    break;
	}
	case EDB_USHORT:
	{
	    unsigned short *us = (unsigned short *)(pp + 
		sizeof(unsigned short) * offst);
	    *us = emp_l;
	    break;
	}
	case EDB_INT:
	{
	    int *ip = (int *)(pp + sizeof(int) * offst);
	    *ip = emp_l;
	    break;
	}
	case EDB_LONG:
	{
	    long int *ip = (long int *)(pp + 
					sizeof(long int) * offst);
	    *ip = emp_l;
	    break;
	}
	case EDB_TIME:
	{
	    time_t *tp = (time_t *)(pp + sizeof(time_t) * offst);
	    *tp = emp_t;
	    break;
	}

	case EDB_FLOAT:
	{
	    float *fp = (float *)(pp + sizeof(float) * offst);
	    *fp = emp_d;
	    break;
	}
	case EDB_DOUBLE:
	{
	    double *dp = (double*)(pp + sizeof(double) * offst);
	    *dp = emp_d;
	    break;
	}
	}
    }
    if (csync_debug > 2) putchar ('\n');
}

void doversion (str)
char *str;
{
    while (*str && isspace(*str))
	str ++;
}

static void parse_object (code, str)
int code;
char *str;
{
    const char *obj_name = rlookup(code, edb_tbl);
    Name2Mapping *np = NULL;
    const Mapping *mp;
    Object *ob;
    Field *fp;
    void *ptr = NULL;
    void *nptr;
    int i;
    char *strstart;

    if (obj_name == NULL)
	obj_name = "UNKNOWN";

    if (csync_debug > 1)
	printf ("Type %s ", obj_name);

    if (code == EDB_VERSION) {
	doversion(str);
	return;
    }
	

    if ((np = find_map (code)) != NULL) {
	if (csync_debug > 2)
	    printf ("Found mapping table for %d = %s\n",
		    code, np -> name);
    }
    else {
	if (csync_debug) printf ("No mapping for %s\n", obj_name);
	return;
    }

    ob = find_object (code);
    if (ob == NULL) {
	if (csync_debug) printf("NO DEFINITION for %d %s\n", code, str);
	return;
    }
    while (*str && isspace(*str))
	str ++;

    /* pass1 - to find the keys etc */
    strstart = str;
    ptr = (void *)calloc (1, np -> ssize);
    for (fp = ob -> field; fp && *str; fp = fp -> next) {
	char *cp = getname(&str);

	for (i = 0; i < fp -> fsize; i++) {
	    if (i != 0)
		cp = getname(&str);
	    if (np && ptr) {
		if (csync_debug > 1)
		    printf ("%s=%s ", fp -> fname, cp);
		if ((mp = find_mapname (fp -> fname, np -> mp)) != NULL) {
		    if (*cp) updatefield (np, fp, mp, ptr, i, cp);
		}
		else if(csync_debug)
		    printf ("No map for %s %s=%s\n", obj_name, fp -> fname,cp);
	    }
	    if (fp -> ftype == EDB_CHAR) /* chars are funny */
		break;
	    if (*str == ',') str ++;
	    if (*str == ':') break;
	}
	if (*str == 0)
	    break;
	else if (*str == ':')
	    str ++;
	else {
	    fprintf (stderr, "ERROR %s", str);
	    break;
	}
    }

    /* now find the real object */
    nptr = (*np -> loc_keyfnx) (ptr);
    free (ptr);
    if (nptr == NULL) return;

    /* and update it */
    str = strstart;
    for (fp = ob -> field; fp && *str; fp = fp -> next) {
	char *cp = getname(&str);

	for (i = 0; i < fp -> fsize; i++) {
	    if (i != 0)
		cp = getname(&str);
	    if (np && nptr) {
		if (csync_debug > 1)
		    printf ("%s=%s ", fp -> fname, cp);
		if ((mp = find_mapname (fp -> fname, np -> mp)) != NULL) {
		    if (*cp)
			updatefield (np, fp, mp, nptr, i, cp);
		}
		else if(csync_debug)
		    printf ("No map for %s %s=%s\n", obj_name, fp -> fname,cp);
	    }
	    if (fp -> ftype == EDB_CHAR) /* chars are funny */
		break;
	    if (*str == ',') str ++;
	    if (*str == ':') break;
	}
	if (*str == 0)
	    break;
	else if (*str == ':')
	    str ++;
	else {
	    fprintf (stderr, "ERROR %s", str);
	    break;
	}
    }
    if (np && np -> loc_finishfnx)
	(*np -> loc_finishfnx) (nptr);
    if (csync_debug > 1) putchar('\n');
}

static int parse_typedef (str)
char *str;
{
    int bnum;
    int onum;
    Object *obj;
    char *name;
    int type;
    int fsize;

    bnum = getnum(&str);
    onum = getnum(&str);
    if(find_object (onum)) return 0;	/* already there */

    obj = new_object(onum);
    obj -> btype = bnum;

    while(*str && isspace(*str))
	str ++;

    while(*str) {
	name = getname(&str);
	if (name == NULL) return -1;
	if (*str == ',') str ++;
	type = getnum(&str);
	if (type == -1) return -1;
	if (*str == ',') {
	    str ++;
	    fsize = getnum(&str);
	}
	else fsize = 1;
	addfield(obj, name, type, fsize);
	while(*str && *str != ':')
	    str ++;
	if (*str == ':') str ++;
    }
    return 0;
}

static int getnum(str)
char **str;
{
    char *p = *str;
    int n;
    int sign;

    while (*p && isspace(*p))
	p++;

    if(!isdigit(*p) && *p != '-') return -1;
    n = 0;
    if (*p == '-') {
	sign = -1;
	p ++;
    }
    else sign = 1;
    while (*p && isdigit (*p))
	n = n * 10 + *p++ - '0';

    *str = p;
    return n * sign;
}


void dumpfield(fp)
Field *fp;
{
    while (fp) {
	const char *cp = rlookup (fp -> ftype, type_tbl);
	if (cp) 
	    printf ("%s=%s", fp -> fname, cp);
	else
	    printf ("%s=<unknown=%d>", fp -> fname, fp -> ftype);
	if (fp -> fsize >1)
	    printf (",%d", fp -> fsize);
	putchar (':');
	fp = fp -> next;
    }
}

void printthing (ptr, map, str)
void *ptr;
const Mapping *map;
const char *str;
{
    char *pp;
    const Mapping *mp;
    int offst;

    printf ("%s:", str);
    for (mp = map; mp -> pc_name; mp++) {
	pp = ptr;
	pp += mp -> xe_offset;
	printf (" %s=", mp -> pc_name);
	for (offst = 0; offst < mp -> xe_number; offst ++) {
	    if (offst > 0) printf (", ");
	    switch (mp -> xe_type) {
	    case EDB_STRING:
	    case EDB_STRARRY:
	    {
		char **cpp = (char **)(pp + sizeof(char *) * offst);
		printf ("\"%s\"", *cpp);
		break;
	    }
	    case EDB_CHAR:
	    {
		char *p = pp + sizeof(char) * offst;
		printf ("%d", *p);
		break;
	    }
	    case EDB_UCHAR:
	    {
		unsigned char *p = (unsigned char *)(pp + 
					     sizeof(unsigned char) * offst);
		printf ("%d", *p);
		break;
	    }
	    case EDB_SHORT:
	    {
		short *ip = (short *)(pp + sizeof(short) * offst);
		printf ("%d", *ip);
		break;
	    }
	    case EDB_USHORT:
	    {
		unsigned short *us = (unsigned short *)(pp + 
							sizeof(unsigned short) * offst);
		printf ("%d", *us);
		break;
	    }
	    case EDB_INT:
	    {
		int *ip = (int *)(pp + sizeof(int) * offst);
		printf ("%d", *ip);
		break;
	    }
	    case EDB_TIME:
	    case EDB_LONG:
	    {
		long int *ip = (long int *)(pp + 
					    sizeof(long int) * offst);
		printf ("%ld", *ip);
		break;
	    }
	    case EDB_FLOAT:
	    {
		float *fp = (float *)(pp + sizeof(float) * offst);
		printf ("%g", *fp);
		break;
	    }
	    case EDB_DOUBLE:
	    {
		double *dp = (double*)(pp + sizeof(double) * offst);
		printf ("%g", *dp);
		break;
	    }
	    }
	}
    }
    putchar('\n');
}

void dumpobj ()
{
    Object *op;

    for (op = Olist; op; op = op -> next) {
	const char *cp = rlookup (op -> otype, edb_tbl);
	if (cp)
	    printf ("Type %s ", cp);
	else
	    printf ("Type <unknown=%d> ", op -> otype);
	dumpfield(op->field);
	putchar ('\n');
    }
}

void dumpmaps ()
{
    Name2Mapping **mpp;

    for (mpp = namemap; *mpp; mpp++)
	((*mpp)->loc_dumpfunc) ();
}

void docopy (k1, k2, obj)
void *k1;
void *k2;
const Mapping obj[];
{
    const Mapping *mp;
    char *sp, *dp;
    char *spbase, *dpbase;
    int offst;

    spbase = k1;
    dpbase = k2;

    for (mp = obj; mp -> pc_name; mp ++) {
	for (offst = 0; offst < mp -> xe_number; offst ++) {
	    sp = spbase + mp -> xe_offset;
	    dp = dpbase + mp -> xe_offset;
	    switch (mp -> xe_type) {
	    case EDB_STRING:
	    {
		char **spp = (char **)(sp + sizeof(char *) * offst);
		char **dpp = (char **)(dp + sizeof(char *) * offst);
		*dpp = *spp;
		break;
	    }
	    case EDB_STRARRY:
	    {
		char **spp = (char **)(sp + sizeof(char *) * offst);
		char **dpp = (char **)(dp + sizeof(char *) * offst);
		strcpy(*dpp,*spp);
		break;
	    }
	    case EDB_CHAR:
	    {
		char *sc = sp + sizeof(char) * offst;
		char *dc = dp + sizeof(char) * offst;
		*dc = *sc;
		break;
	    }
	    case EDB_UCHAR:
	    {
		unsigned char *su = (unsigned char *)
		    (sp + sizeof(unsigned char) * offst);
		unsigned char *du = (unsigned char *)
		    (dp + sizeof(unsigned char) * offst);
		*du = *su;
		break;
	    }
	    case EDB_SHORT:
	    {
		short *si = (short *)(sp + sizeof(short) * offst);
		short *di = (short *)(dp + sizeof(short) * offst);
		*di = *si;
		break;
	    }
	    case EDB_USHORT:
	    {
		unsigned short *su = (unsigned short *)
		    (sp + sizeof(unsigned short) * offst);
		unsigned short *du = (unsigned short *)
		    (dp + sizeof(unsigned short) * offst);
		*du = *su;
		break;
	    }
	    case EDB_INT:
	    {
		int *si = (int *)(sp + sizeof(int) * offst);
		int *di = (int *)(dp + sizeof(int) * offst);
		*di = *si;
		break;
	    }
	    case EDB_TIME:
	    case EDB_LONG:
	    {
		long int *sl = (long int *)(sp + 
					    sizeof(long int) * offst);
		long int *dl = (long int *)(dp + 
					    sizeof(long int) * offst);
		*dl = *sl;
		break;
	    }
	    case EDB_FLOAT:
	    {
		float *sf = (float *)(sp + sizeof(float) * offst);
		float *df = (float *)(dp + sizeof(float) * offst);
		*df = *sf;
		break;
	    }
	    case EDB_DOUBLE:
	    {
		double *sd = (double*)(sp + sizeof(double) * offst);
		double *dd = (double*)(dp + sizeof(double) * offst);
		*dd = *sd;
		break;
	    }
	    }
	}
    }
}
