/*
 * $Header: /a/victor/nexor/user4/jpo/xemp/xemp5.0/lib/csync/RCS/land.c,v 1.3 1996/07/19 08:24:08 jpo Exp $
 *
 * $Log: land.c,v $
 * Revision 1.3  1996/07/19 08:24:08  jpo
 * Track land movements
 *
 * Revision 1.2  1996/07/17 06:58:35  jpo
 * New format table
 *
 */

#include "toolkit.h"
#include "jsync.h"
#include "main.h"
#include "land.h"
#include "sector.h"

extern struct l_landtype landtypes[];
extern int csync_debug;

static void *land_class_key (void *key) 
{
    int i;
    struct l_landtype *pbase = key;

    for (i = 0; i < MAX_LANDTYPES; i++) {
	if (landtypes[i].nam == 0 ||
	    strcasecmp(landtypes[i].nam, pbase->nam) == 0)
	    break;
    }
    return &landtypes[i];
}


static Mapping ld_obj[] = {
    {"owner", 1, EDB_SHORT, offsetof(struct l_land, owner) },
    {"uid", 1, EDB_SHORT, offsetof(struct l_land, nr) },
    {"xloc", 1, EDB_SHORT, offsetof(struct l_land, x) },
    {"yloc", 1, EDB_SHORT, offsetof(struct l_land, y) },
    {"type", 1, EDB_SHORT, offsetof(struct l_land, type) },
    {"effic", 1, EDB_SHORT, offsetof(struct l_land, eff) },
    {"mobil", 1, EDB_SHORT, offsetof(struct l_land, mob) },
/*    {"sell", 1, EDB_INT, offsetof(struct l_land, mob) }, */
    {"tech", 1, EDB_SHORT, offsetof(struct l_land, tech) },
/*    {"group", 1, EDB_INT, offsetof(struct l_land, tech) }, */
    {"opx", 1, EDB_SHORT, offsetof(struct l_land, rtrtx) },
    {"opy", 1, EDB_SHORT, offsetof(struct l_land, rtrty) },
    {"mission", 1, EDB_CHAR, offsetof(struct l_land, mission) },
    {"army", 1, EDB_CHAR, offsetof(struct l_land, army) },
    {"ship", 1, EDB_INT, offsetof(struct l_land, shipnr) },
    {"harden", 1, EDB_SHORT, offsetof(struct l_land, fort) },
/*    {"retreat", 1, EDB_UCHAR, offsetof(struct l_land, range) }, */
    {"fuel", 1, EDB_UCHAR, offsetof(struct l_land, fuel) },
    {"nxlight", 1, EDB_UCHAR, offsetof(struct l_land, xpln) },
    {"civil", 1, EDB_UCHAR, offsetof(struct l_land, civ) },
    {"milit", 1, EDB_UCHAR, offsetof(struct l_land, mil) },
    {"shell", 1, EDB_CHAR, offsetof(struct l_land, shell) },
    {"gun", 1, EDB_UCHAR, offsetof(struct l_land, gun) },
    {"petrol", 1, EDB_UCHAR, offsetof(struct l_land, pet) },
    {"iron", 1, EDB_UCHAR, offsetof(struct l_land, iron) },
    {"dust", 1, EDB_UCHAR, offsetof(struct l_land, dust) },
    {"bar", 1, EDB_UCHAR, offsetof(struct l_land, bar) },
    {"food", 1, EDB_UCHAR, offsetof(struct l_land, food) },
    {"oil", 1, EDB_UCHAR, offsetof(struct l_land, oil) },
    {"lcm", 1, EDB_UCHAR, offsetof(struct l_land, lcm) },
    {"hcm", 1, EDB_UCHAR, offsetof(struct l_land, hcm) },
    {"uw", 1, EDB_UCHAR, offsetof(struct l_land, uw) },
    {"rad", 1, EDB_UCHAR, offsetof(struct l_land, rad) },
    { NULL }
};

static Mapping ld_class[] = {
    {"name", 1, EDB_STRING, offsetof (struct l_landtype, nam) },
    {"lcm", 1, EDB_INT, offsetof (struct l_landtype, lcm) },
    {"hcm", 1, EDB_INT, offsetof (struct l_landtype, hcm) },
    {"mil", 1, EDB_INT, offsetof (struct l_landtype, mil) },
    {"gun", 1, EDB_INT, offsetof (struct l_landtype, guns) },
    {"shl", 1, EDB_INT, offsetof (struct l_landtype, shells) },
    {"tech", 1, EDB_INT, offsetof (struct l_landtype, tec) },
    {"cost", 1, EDB_INT, offsetof (struct l_landtype, cos) },
    {"att", 1, EDB_FLOAT, offsetof(struct l_landtype, att) },
    {"def", 1, EDB_FLOAT, offsetof(struct l_landtype, def) },
    {"vul", 1, EDB_INT, offsetof(struct l_landtype, vul) },
    {"spd", 1, EDB_INT, offsetof(struct l_landtype, spd) },
    {"vis", 1, EDB_INT, offsetof(struct l_landtype, vis) },
    {"spy", 1, EDB_INT, offsetof(struct l_landtype, spy) },
    {"rad", 1, EDB_INT, offsetof(struct l_landtype, rad) },
    {"frg", 1, EDB_INT, offsetof(struct l_landtype, frg) },
    {"acc", 1, EDB_INT, offsetof(struct l_landtype, acc) },
    {"dam", 1, EDB_INT, offsetof(struct l_landtype, dam) },
    {"ammo", 1, EDB_INT, offsetof(struct l_landtype, amm) },
    {"aaf", 1, EDB_INT, offsetof(struct l_landtype, aaf) },
    {"fuelc", 1, EDB_INT, offsetof(struct l_landtype, fcar) },
    {"fuelu", 1, EDB_INT, offsetof(struct l_landtype, fuse) },
    {"nxlight", 1, EDB_INT, offsetof(struct l_landtype, xpl) },
    {"flags", 1, EDB_LONG, offsetof(struct l_landtype, flags) },
    {"civil", 1, EDB_INT, offsetof(struct l_landtype, car[V_CIVIL]) },
    {"milit", 1, EDB_INT, offsetof(struct l_landtype, car[V_MILIT]) },
    {"shell", 1, EDB_INT, offsetof(struct l_landtype, car[V_SHELL]) },
    {"gun", 1, EDB_INT, offsetof(struct l_landtype, car[V_GUN]) },
    {"petrol", 1, EDB_INT, offsetof(struct l_landtype, car[V_PETROL]) },
    {"iron", 1, EDB_INT, offsetof(struct l_landtype, car[V_IRON]) },
    {"dust", 1, EDB_INT, offsetof(struct l_landtype, car[V_DUST]) },
    {"bar", 1, EDB_INT, offsetof(struct l_landtype, car[V_BAR]) },
    {"food", 1, EDB_INT, offsetof(struct l_landtype, car[V_FOOD]) },
    {"oil", 1, EDB_INT, offsetof(struct l_landtype, car[V_OIL]) },
    {"lcm", 1, EDB_INT, offsetof(struct l_landtype, car[V_LCM]) },
    {"hcm", 1, EDB_INT, offsetof(struct l_landtype, car[V_HCM]) },
    {"uw", 1, EDB_INT, offsetof(struct l_landtype, car[V_UW]) },
    {"rad", 1, EDB_INT, offsetof(struct l_landtype, car[V_RAD]) },

    { NULL }
};


static void *land_obj_key (key)
void *key;
{
    Land land;
    Land nland = (Land)key;

    land = NrToLand (ld_nr(nland));
    if (land == NULL) {
	land = (Land) malloc (sizeof *land);
	*land = *nland;
	AddLandToList(land);
	return land;
    }
    DeleteLandFromSector(land);
    DrawSector(World(ld_xcd(land), ld_ycd(land), S_DESIG));
    return land;
}

static void land_obj_final(ptr)
void *ptr;
{
    Land land = ptr;

    set_ld_marked(land, False);
    AddLandToSector (land, World (ld_xcd(land), ld_ycd(land), S_DESIG));
    DrawSector (World (ld_xcd(land), ld_ycd(land), S_DESIG));
    return;
}

static void dump_land_class ()
{
    int i;
    for (i = 0; landtypes[i].nam != NULL; i++)
	printthing (&landtypes[i], ld_class, "Land class");
}

static void dump_land_obj ()
{
    Land ptr;

    for ALL_LANDS(ptr)
	printthing (ptr, ld_obj, "Land object");
}


Name2Mapping land_obj_map =
{"land object", LND_OBJECT, sizeof(struct l_land),
 ld_obj, land_obj_key, land_obj_final, dump_land_obj };

Name2Mapping land_class_map =
{"land class", LND_CLASS, sizeof(struct l_landtype), 
 ld_class, land_class_key, NULL, dump_land_class};
