/*
	$Header: /a/victor/nexor/user4/jpo/xemp/xemp5.0/lib/land/RCS/retreat.c,v 1.6 1996/04/23 07:47:57 jpo Exp $
	$Date: 1996/04/23 07:47:57 $
	$Author: jpo $
	$Id: retreat.c,v 1.6 1996/04/23 07:47:57 jpo Exp $
	$Locker:  $
	$Log: retreat.c,v $
	Revision 1.6  1996/04/23 07:47:57  jpo
	New API

	Revision 1.5  1996/03/29 09:54:23  jpo
	Lint fixups

	Revision 1.4  1996/01/29 08:17:30  jpo
	New interface for selections

	Revision 1.3  1995/09/10 17:39:58  jpo
	Add army support and display

	Revision 1.2  1995/09/08 07:20:27  jpo
	retreat stuff

 * Revision 1.1  93/03/14  16:48:27  etienne
 * Initial revision
 * 
*/

#include "main.h"
#include "version.h"
#include "sector.h"
#include "land.h"
#include "func.h"

#define HOW_SELECT	0	/* via menu */
#define HOW_CURRENT	1	/* via redo */
#define HOW_BOTH	2	/* via bind */

extern const char *retreatcond[];
extern const char retreat_n2cond[];

static bool SetLandRetreat _PROTO((const char *lands,
				uchar cond,
				Sector sct,
				short *rtrxp, short *rtryp,
				uchar *condp));

static bool SetLandRetreat (lands, cond, sct, rtrxp, rtryp, condp)
const char *lands;
uchar cond;
Sector sct;
short *rtrxp, *rtryp;
uchar *condp;
{
	Strings strings;
	char buf[10];
	char *cp;
	int nr;
	Sector to;
	int i;
	char *path;

	strings = InitStrings ();
	AddStringsID (strings, retreatcond,  RETREAT_MAXSTR, 0);
	MakeStringIndex (strings);
	for (i = 0; i < RETREAT_MAX; i++)
		if (cond & (1 << i))
			SetIthFlag (strings, i, 1);
	
	switch (MultiSelectPager (strings, "Retreat Conditions",
				  "Delete", DELETEFUNC, NULL)) {
	case DELETEFUNC:
		FreeStrings (strings);
		FeedCommand (Fmt ("lretreat %s h C", lands),
			     PRINT);
		*condp = RETREAT_NONE;
		return True;

	case CANCELFUNC:
	default:
		FreeStrings (strings);
		return False;

	case DONEFUNC:
		break;
	}
	InitStringList (strings);
	cp = buf;
	cond = RETREAT_NONE;
	while ((nr = GetNextSelectedId (strings)) != -1) {
		*cp ++ = retreat_n2cond[nr];
		cond |= (1 << nr);
	}
	*cp = '\0';
	FreeStrings (strings);

	Message (Fmt ("Retreat land %s to sector?", lands));
	to = ChooseLandDest (sct, &path);
	if (to == NULL) {
		return False;
	}
	*condp = cond;
	*rtrxp = s_xcd (to);
	*rtryp = s_ycd (to);
	FeedCommand ( Fmt ("lretreat %s %s %s", lands, path, buf),
		      PRINT);
	return True;
}
 


void LRetreat (list)
const char *list;
{
	short rx, ry;
	uchar cond;
	Land ptr, aland;

	UnmarkAllLands ();
	MarkLands (list);
	if (LMarkedColocated () == False) {
	    Message ("Units are not colocated");
	    return;
	}
	aland = NULL;
	for ALL_LANDS(ptr) {
	    if (ld_marked(ptr)) {
		if (aland == NULL) {
		    aland = ptr;
		    cond = ptr -> rtrcond;
		}
		else if (cond != ptr -> rtrcond)
		    cond = 0;
	    }
	}
	if (aland == NULL) {
	    Message ("No units selected");
	    return;
	}

	if (SetLandRetreat (list, cond,
			    World (ld_xcd(aland), ld_ycd(aland), S_DESIG),
			    &rx, &ry,
			    &cond) == False)
	    return;

	for ALL_LANDS(ptr) {
	    if (ld_marked(ptr)) {
		ptr -> rtrcond = cond;
		ptr -> rtrtx = rx;
		ptr -> rtrty = ry;
	    }
	}
}


void BindLandRetreat (x, y, str)
int x, y;
char *str;
{
	static char buf [100];

	if (sscanf (str, " ( %s ) ", buf) == 1) {
		LRetreat (buf);
	}
	else
		DoLandRetreatOrder (x, y, HOW_BOTH);
}

void LandRetreat (x, y)
int x, y;
{
	DoLandRetreatOrder (x, y, HOW_SELECT);
}

void DoLandRetreatOrder (x, y, how)
int x, y;
int how;
{
    	char * ptr;

	if (last_layout == LAND_MODE && how != HOW_SELECT)
	{
		if (! ld_owned (curland))
		{
			Message ("Not your unit!");
			Bell ();
			return;
		}

		LRetreat (Fmt("%d", ld_nr(curland)));
	}
	else if (last_layout == ARMY_MODE && how != HOW_SELECT)
		LRetreat (Fmt("%c", curarmy));
	else
	{
		if (how == HOW_CURRENT)
		{
			Message ("No current army/unit");
			Bell ();
			return;
		}

		ptr = SelectLands (x, y, LD_ORDERS, "Retreat which units ?");
		if (ptr)
			LRetreat (ptr);
	}
}

void FScanLandRetreat (list, flag)
const char *list;
int flag;
{
	FeedEmpire (Fmt ("lretreat %s", list), flag);

	ScanLandRetreat (flag);
	(void) WaitForPrompt (flag);
}

void ScanLandRetreat (flag)
int flag;
{
	char *ptr, *ind;
	Land land;
	int sno;
	int flagoff, pathoff;

	if (EmpireStatus () != E_PRINTING)
		return;
	if (flag == PRINT)
		PrintAtEmpire ("Empiretool: scanning land retreats");
	
	ptr = ReadEmpire (DONT_PRINT);
	if ((ind = StrStr (ptr, "unit type")) == NULL) {
		PrintAtEmpire (Fmt ("Order scan failed on %s", ptr));
		return;
	}
	if ((ind = StrStr (ptr, "path")) == NULL) {
		PrintAtEmpire (Fmt ("No path in %s", ptr));
		return;
	}
	pathoff = ind - ptr;

	if ((ind = StrStr (ptr, "flags")) == NULL) {
		PrintAtEmpire (Fmt ("No flags in %s", ptr));
		return;
	}
	flagoff = ind - ptr;
	for (;;) {
		Sector to;
		ind = ptr = ReadEmpire (DONT_PRINT);

		if (EmpireStatus () != E_PRINTING) {
			if (flag == PRINT)
				PrintAtEmpire (ptr);
			return;
		}
		
		if (ScanDigit ((ConstVP)&ind, &sno)) /* land name? */
			continue;
		if (strncmp (ind, "land", 4) == 0) /* %d lands */
			break;
		if (strlen(ptr) < flagoff)
			continue;
		if ((land = NrToLand (sno)) == (Land)0) {
			PrintAtEmpire (Fmt ("#%d Unknown unit (rescan lands!)",
					    sno));
			Bell ();
			(void) EmpireMore ();
			continue;
		}
		land -> rtrcond = RETREAT_NONE;
		for (ind = &ptr[flagoff]; *ind; ind ++) {
			char *cp;
			if ((cp = index (retreat_n2cond, *ind)) == NULL)
				continue;
			land -> rtrcond |= (1 << (cp - retreat_n2cond));
		}
		ind = &ptr[pathoff];
		to = FollowPath (World(ld_xcd(land), ld_ycd(land), S_DESIG),
				 ind);
		land -> rtrtx = s_xcd (to);
		land -> rtrty = s_ycd (to);
	}
}



Sector ChooseLandDest (sct, pathp)
Sector sct;
char **pathp;
{
	int button;
	Sector to;

	MesBut ("View", "Cancel", "Set");
	for (;;) {
		to = SelectSector (&button);
		
		if (interrupt || button == MIDDLE_BUTTON) {
			interrupt = False;
			Message ("Cancelled");
			MesClearBut ();
			return NULL;
		}
		if (to == (Sector)0)
			continue;

		if (button == LEFT_BUTTON) {
			CensusSct (to);
			continue;
		}
		if (to == sct)
			continue;
		if (s_mines (to) > 0 || s_des(to) == '.') {
			Message ("Not a good navigable sector!");
			continue;
		}
		if ((*pathp = BestTmpDistPath (sct, to)) == NULL) {
			Message ( Fmt("I don't see a path from %s to %s",
				      CrdStr(sct), CrdStr(to)));
			return NULL;
		}
		MesClearBut ();
		return to;
	}
}

const char *ArmyHasRetreat (army)
char army;
{
	Land ptr;
	uchar rcond = 0;
	int first = 0;

	for (ptr = landlist; ptr != (Land) 0; ptr = ptr-> next)
		if (ptr-> army == army) {
			if (first++ == 0) 
				rcond = ld_retreat(ptr);
			else if (rcond != ld_retreat(ptr))
				return "Mixed";
		}
	
	if (rcond == RETREAT_NONE)
		return "No";
	return RtrtList (rcond);
}
