/*
    $Header: /a/victor/nexor/user4/jpo/xemp/xemp5.0/lib/misc/RCS/highl.c,v 5.5 1996/07/17 07:46:39 jpo Exp $
    $Date: 1996/07/17 07:46:39 $
    $Author: jpo $
    $Id: highl.c,v 5.5 1996/07/17 07:46:39 jpo Exp $
    $Locker:  $
    $Log: highl.c,v $
    Revision 5.5  1996/07/17 07:46:39  jpo
    Deal with partial saves

    Revision 5.4  1996/03/29 09:53:42  jpo
    Lint fixups

    Revision 5.3  1996/01/29 08:18:47  jpo
    New selection interface

    Revision 5.2  1995/09/08 07:27:57  jpo
    added new temporary highlight command
    redone the default ones so they make sense

 * Revision 5.1  93/03/14  16:49:19  etienne
 * *** empty log message ***
 * 
 * Revision 5.0  93/02/06  09:18:49  greyhelm
 * y
 * Fixed backward compatabilty with Merc/KSU
 * Changed MOTD to show new version and authors
 * 
 * Revision 4.4  1993/02/06  04:37:47  greyhelm
 * Added RCS headers - Karl Hagen
 *

*/
#include "type.h"
#include "main.h"
#include "sector.h"
#include "func.h"
#include "var.h"
#include "ship.h"
#include "parser.h"

#define NR_HIGHLSTR	10

static const char *highl_str [NR_HIGHLSTR];

static Stack cur_highl;

void InitHighlight ()
{
	highl_str [0] = Str ("threshold(food) == 0 && des=w");
	highl_str [1] = Str ("t(food) == 0");
	highl_str [2] = Str ("iron > 1");
	highl_str [3] = Str ("food < 25");
	highl_str [4] = Str ("r(oil) > 20");
	highl_str [5] = Str ("lcm > 1");
	highl_str [6] = Str ("hcm > 1");
	highl_str [7] = Str ("resource(dust) > 20");
	highl_str [8] = Str ("r(iron) > 80");
	highl_str [9] = Str ("r(ura) > 20  ; Nuke 'em all to death!");

	cur_highl = (Stack) 0;
}

void ReadHighlight (fp)
FILE *fp;
{
    char buffer [BUFSIZ];
    int i;

    for (i = 0; ; i++)
	if (fgets (buffer, BUFSIZ, fp) != (char *) 0)
	{
	    if (strncmp (buffer, "END OF HIGHLIGHT LIST", 21) == 0)
		break;

	    buffer [strlen (buffer) - 1] = '\0';	/* no newline */

	    if (i < NR_HIGHLSTR) {
		if (highl_str [i] != (char *) 0)
		    free ((char *)highl_str [i]);
		highl_str [i] = Str (buffer);
	    }
	}
	else
	    break;
	
    while (i < NR_HIGHLSTR)
	if (highl_str [i] == (char *) 0)
	    highl_str[i++] = Str ("");

    cur_highl = (Stack) 0;
}

void SaveHighlight (fp)
FILE *fp;
{
    int i;

    for (i = 0; i < NR_HIGHLSTR; i++) {
	if (highl_str[i] != NULL)
	    fprintf (fp, "%s\n", highl_str [i]);
    }

    fprintf (fp, "END OF HIGHLIGHT LIST\n");
}

bool HighlightSector (sct)
Sector sct;
{
	if (cur_highl == (Stack) 0)
		return False;
	
	return EvaluateExpression (sct, cur_highl) != 0.0;
}

struct hdata {
	int x, y;
};

static void SelectFnx (strings, ind, data)
Strings strings;
int ind;
void *data;
{
	const char *str = GiveIdString (strings, ind);
	Stack tmp;

	tmp = ParseExpression (str);
	if (tmp != (Stack)0) {
		DestroyExpression (&cur_highl);
		cur_highl = tmp;
	}
	DrawMap ();
}

static void EditFnx (strings, ind, data)
Strings strings;
int ind;
void *data;
{
	char *ans;
	Stack expr;

	do {
		ans = InputAtMessage ("Expr: ", 60, GS_TEXT);
		if (ans == (char *)0 || interrupt || *ans == '\0') {
			Message ("cancelled");
			return;
		}
		expr = ParseExpression (ans);
	} while (expr == (Stack)0);

	ChangeString (strings, ind, ans);
	DestroyExpression (&cur_highl);
	cur_highl = expr;
	DrawMap ();
}


static FuncList high_fnxs[] = {
	{ "Select", SELECTFUNC, SelectFnx },
	{ "No highlight", NONEFUNC, NULL },
	{ "Cancel", DONEFUNC, NULL },
	{ "Change", NEWFUNC, EditFnx },
	{ NULL},
};

#define         HL_MAX_X        70

void SetHighlight ()
{
	Strings strings;
	int i;
	const char *ptr;
	int func;
	extern WinInfo empire_win;
	struct hdata hdata;

	hdata.x = 0;
	hdata.y = 0;
	strings = InitStrings ();
	AddStringsID (strings, (ConstVP)highl_str, NR_HIGHLSTR, 0);
	func = MakePickDialog (strings,
			       Fmt ("%-*.*s", HL_MAX_X - 4, HL_MAX_X - 4,
				    "New highlight ?"),
			       high_fnxs, &hdata, empire_win, 0, 0, False);

	InitStringList (strings);
	for (i = 0; (ptr = GetNextString(strings)) != NULL; i++) {
	    free((char *)highl_str[i]);
	    highl_str[i] = Str(ptr);
	}
	
	FreeStrings (strings);

	switch (func) {
	case NONEFUNC:
		DestroyExpression (&cur_highl);
		DrawMap ();
		break;

	default:
		break;
	}
}

void ExprHighlight (expr)
Stack expr;
{
	if (expr != (Stack) 0)
	{
		DestroyExpression (& cur_highl);
		cur_highl = expr;
	}
}
