/*
	$Header: /a/victor/nexor/user4/jpo/xemp/xemp5.0/lib/plane/RCS/mission.c,v 1.6 1996/04/26 08:19:51 jpo Exp $
	$Date: 1996/04/26 08:19:51 $
	$Author: jpo $
	$Id: mission.c,v 1.6 1996/04/26 08:19:51 jpo Exp $
	$Locker:  $
	$Log: mission.c,v $
	Revision 1.6  1996/04/26 08:19:51  jpo
	Redone the API completely

	Revision 1.5  1996/03/29 09:45:50  jpo
	Lint fixups

	Revision 1.4  1996/01/29 08:27:21  jpo
	New menu stuff

	Revision 1.3  1995/09/08 07:40:06  jpo
	const

	Revision 1.2  1995/08/15 08:05:26  jpo
	lots of mission changes

 * Revision 1.1  93/03/14  16:51:12  etienne
 * Initial revision
 * 
*/

#include "main.h"
#include "sector.h"
#include "version.h"
#include "plane.h"

static const char *mission_types[] = {
	"Delete Mission",
#define MIS_DELETE	0
	"Interdict",
#define MIS_INTERDICT	1
	"Support",
#define MIS_SUPPORT	2
	"Offensive Support",
#define MIS_OFFSUPP	3
	"Defensive Support",
#define MIS_DEFSUPP	4
	"Escort",
#define MIS_ESCORT	5
	"Air Defense",
#define MIS_AIRDEF	6
	(char *)0
};
#define MISSION_MAX	7

#define HOW_SELECT	0	/* via menu */
#define HOW_CURRENT	1	/* via redo */
#define HOW_BOTH	2	/* via bind */

static const char type2mission[] = "\0isodea";

static bool SetPlaneMission (list, range, sct, missp, rangep)
const char *list;
int range;
Sector sct;
char *missp;
int *rangep;
{
	int i;
	Strings strings;
	char *answer;
	int mset = 0;
	Plane ptr;

	strings = InitStrings ();
	AddStringID (strings, mission_types[MIS_DELETE], MIS_DELETE);
	for ALL_PLANES(ptr) {
		if (!pl_marked(ptr)) continue;
		if ((mset & P_T) == 0 && PlaneHas (ptr, P_T)) {
			mset |= P_T;
			AddStringID (strings, mission_types[MIS_INTERDICT],
				     MIS_INTERDICT);
			AddStringID (strings, mission_types[MIS_SUPPORT],
				     MIS_SUPPORT);
			AddStringID (strings, mission_types[MIS_OFFSUPP],
				     MIS_OFFSUPP);
			AddStringID (strings, mission_types[MIS_DEFSUPP],
				     MIS_DEFSUPP);

		}
		if ((mset & P_E) == 0 &&
		    (PlaneHas(ptr, P_E) || PlaneHas(ptr, P_F))) {
			mset |= P_E;
			AddStringID (strings, mission_types[MIS_ESCORT],
				     MIS_ESCORT);
		}
		if ((mset & P_F) == 0 && PlaneHas(ptr, P_F)) {
			mset |= P_F;
			AddStringID (strings, mission_types[MIS_AIRDEF],
				     MIS_AIRDEF);
		}
	}
	i = ChooseMenu (strings, "Mission Type?", map_win, 5, 5, NULL);
	FreeStrings (strings);

	if (i < 0 || interrupt) {
		interrupt = False;
		Message ("Set mission cancelled");
		return False;
	}
	
	if (i == MIS_DELETE) {
		*missp = type2mission[i];
		*rangep = 0;
		FeedCommand (Fmt ("mission plane %s c", list), PRINT);
		return True;
	}

	answer = InputAtMessage (Fmt  ("Range [%d]: ",range),
				 3, GS_PNUMBER);
	if (interrupt || answer == (char *)0) {
		interrupt = False;
		Message ("Mission cancelled");
		return False;
	}
	if (*answer != '\0')
		range = atoi (answer);
		
	*rangep = range;
	*missp = type2mission[i];
	FeedCommand (Fmt ("mission plane %s %c %s %d",
			  list, *missp,
			  CrdStr(sct), range), PRINT);
	return True;
}

void PMission (list)
const char *list;
{
	int range = 0;
	int r;
	char miss;
	Plane ptr, aplane;

	UnmarkAllPlanes ();
	MarkPlanes (list);

	aplane = NULL;
	for ALL_PLANES(ptr) {
		if (pl_marked(ptr)) {
			r = pl_range(ptr);
			if (r > range)
				range = r;
			aplane = ptr;
		}
	}
	if (aplane == NULL) {
		Message ("No planes selected");
		return;
	}

	if (SetPlaneMission (list, range, PlaneSector(aplane),
			     &miss, &range) == False)
		return;
	for ALL_PLANES(ptr) {
		if (pl_marked(ptr))
			set_pl_mission(ptr, miss);
	}
}

void PlaneMission (x, y)
int x, y;
{
	DoPlaneMission (x, y, HOW_SELECT);
}

void DoPlaneMission (x, y, how)
int x, y;
int how;
{
    	char * ptr;

	if (last_layout == PLANE_MODE && how != HOW_SELECT)
	{
		if (! pl_owned (curplane))
		{
			Message ("Not your plane!");
			Bell ();
			return;
		}

		PMission (Fmt("%d", pl_nr(curplane)));
	}
	else if (last_layout == WING_MODE && how != HOW_SELECT)
		PMission (Fmt("%c", curwing));
	else
	{
		if (how == HOW_CURRENT)
		{
			Message ("No current plane/wing");
			Bell ();
			return;
		}

		ptr = SelectPlanes (x, y, cursct, MIS_ORDER,
				   "Missions for which planes ?");
		if (ptr)
			PMission (ptr);
	}
}


void FScanPlaneMission (list, flag)
const char *list;
int flag;
{
	FeedEmpire (Fmt ("mission plane %s q", list), flag);

	ScanPlaneMission (flag);
	(void) WaitForPrompt (flag);
}

void ScanPlaneMission (flag)
int flag;
{
	char *ptr, *ind;
	int mind, pno;
	Plane plane;

	if (EmpireStatus () != E_PRINTING)
		return;
	if (flag == PRINT)
		PrintAtEmpire ("Empiretool: scanning plane missions");

	ptr = ReadEmpire (DONT_PRINT);
	if ((ind = StrStr (ptr, "mission")) == NULL) {
		PrintAtEmpire (Fmt ("Mission scan failed on %s", ptr));
		return;
	}
	mind = ind - ptr;
	
	for (;;) {
		ptr = ReadEmpire (DONT_PRINT);

		if (EmpireStatus () != E_PRINTING) {
			if (flag == PRINT)
				PrintAtEmpire (ptr);
			return;
		}
		
		if ((ind = index (ptr, '#')) == NULL)
			continue;

		ind ++;
		if (!ScanDigit ((ConstVP)&ind, &pno) ||
		    (plane = NrToPlane (pno)) == (Plane)0) {
			PrintAtEmpire (Fmt ("#%d Unknown plane (rescan planes!)",
					    pno));
			Bell ();
			(void) EmpireMore ();
			continue;
		}

		switch (ptr[mind]) {
		    case 'i':
		    case 's':
		    case 'o':
		    case 'd':
		    case 'e':
		    case 'a':
			set_pl_mission (plane, ptr[mind]);
			break;
		    case 'h':
			set_pl_mission (plane, '\0');
			break;
		    default:
			PrintAtEmpire (Fmt ("Unknown mission type %s",
					    ptr + mind));
			Bell ();
			break;
		}
	}
}
